'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.

Note that pyglet users should probably just add the data directory to the
pyglet.resource search path.
'''

import os
import pygame

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))

def create_relpath(*args):
    return os.path.join(*args)

def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(data_dir, filename), mode)

class Surfaces(object):
    instance = None
    def __init__(self):
        self.surfaces = {}
        Surfaces.instance = self

    @staticmethod
    def get_instance():
        if not Surfaces.instance:
            raise RuntimeError("Instance requested before initialisation")
        return Surfaces.instance

    def get(self, path):
        if not self.surfaces.has_key(path):
            f = load(path)
            surface = pygame.image.load(f, path)
            f.close()
            surface.convert_alpha()
            self.surfaces[path] = surface
        return self.surfaces[path]


