
import pygame
from drawable import Drawable
from updateable import Updateable

class Animateable(Drawable, Updateable):
    def __init__(self, pos, sprite, animations, default_action, direction):
        Drawable.__init__(self, pygame.Rect(0, 0, 10, 10))
        self._pos = pos
        self._sprite = sprite
        self._active_anim = pygame.Rect(0, 0, 10, 10)
        self._animations = animations
        self._action = default_action
        self._direction = direction

        self._frame = 0
        self._time = 0
        self._frame_time = self._get_time(self._action)
        self._frame_count = self._get_data(self._action, "count")

        self.animate()

    def get_direction(self):
        return self._direction

    def _get_data(self, action, prop):
        try:
            ret = self._animations[action][prop]
        except KeyError:
            ret = self._animations["default"][prop]
        return ret

    def _get_time(self, action):
        base_time = self._get_data(action, 'time')
        return base_time

    def get_animation_time(self, action):
        return self._get_time(action) * (self._get_data(action, 'count') - 1)

    def set_action(self, action):
        self._action = action
        self._frame_time = self._get_time(self._action)
        self._frame_count = self._get_data(self._action, "count")

    def get_action(self):
        return self._action

    def set_direction(self, direction):
        self._direction = direction

    def reset_animation(self):
        self._frame = 0
        self._time = 0

    def update(self, delta):
        self._time += delta
        if self._time >= self._frame_time:
            self._time -= self._frame_time
            self._frame += 1
            #self._frame %= self._frame_count
            if self._frame >= self._frame_count:
                self._frame -= self._frame_count
                self.animation_end(self._action)

        self.animate()

    def animate(self):
        start = self._get_data(self._action, 'start')
        size = self._get_data(self._action, 'size')
        count = self._get_data(self._action, 'count')
        index_y = self._get_data(self._action, self._direction)
        time = self._get_data(self._action, "time")
        offset = self._get_data(self._action, "offset")

        self._active_anim.width, self._active_anim.height = size
        self._active_anim.x = start[0] + self._frame * size[0]
        self._active_anim.y = start[1] + index_y * size[1]

        self.draw_pos.x = self._pos.x - offset[0]
        self.draw_pos.y = self._pos.y - offset[1]

    def draw(self, surface, viewport):
        Drawable.draw(self, surface, viewport)
        #surface.fill((0, 127, 0), self.draw_pos)
        #surface.fill((0, 0, 127), self._active_anim)
        #surface.fill((127, 0, 0), self.pos)
        surface.blit(self._sprite, self.draw_pos, self._active_anim)
        #surface.blit(self._sprite, self.draw_pos)
        
    def animation_end(self, action):
        # to be implemented by subclass. is called every time a animation has finished playing (eg attacking)
        pass

    def frame_last(self):
        self._frame = 5
