import pygame
import pygame.locals as pgl
import os

import lights

from drawable import Drawable
from updateable import Updateable
from eventable import Eventable
from collideable import Collideable
from animateable import Animateable

from random import choice

class WeaponChooser(Drawable):
    def __init__(self, surfaces):
        self.weapons = [
                ["dagger", surfaces.get("daggerx2.png")],
                #["bow", surfaces.get("bowx2.png")],
                ["spear", surfaces.get("spearx2.png")],
                ["sword", surfaces.get("swordx2.png")],
                ]

        self.bg_inactive = surfaces.get("weapon_inactive.png")
        self.bg_active = surfaces.get("weapon_active.png")

        self.active_weapon = 0
        self.weapon_count = len(self.weapons)

    def next_weapon(self):
        self.active_weapon += 1
        self.active_weapon %= self.weapon_count

    def prev_weapon(self):
        self.active_weapon -= 1
        self.active_weapon %= self.weapon_count

    def draw(self, screen, viewport = None):
        for i, w in enumerate(self.weapons):
            offset = (i * 4 + i * self.bg_inactive.get_width() + 4, 4)
            bg = self.bg_active if i == self.active_weapon else self.bg_inactive
            screen.blit(bg, offset)
            screen.blit(w[1], (offset[0] + 4, offset[1] + 4))

    def get_active_weapon(self):
        return self.weapons[self.active_weapon][0]


#class Player(Drawable, Updateable, Eventable, Collideable):
class Player(Animateable, Eventable, Collideable):
    def __init__(self, game, pos, state = None):
        self.game = game
        draw = pygame.Rect(pos, (64, 64))
        pos = pygame.Rect(pos, (32, 32))
        #Drawable.__init__(self, draw)
        Collideable.__init__(self, pos, state)
        surfaces = game.get_surfaces()
        self.surfaces = surfaces
        spritesheet_path = os.path.join("character", "lpc.png")
        #self.spritesheet = surfaces.get(self.spritesheet_path)
        spritesheet = surfaces.get(spritesheet_path)
        self.spritesheet = spritesheet

        self.direction = False
        self.last_direction = "down"

        #self.game_pos = pygame.Rect(0, 0, 32, 32)
        #self.draw_pos = pygame.Rect(0, 0, 32, 32)
        self.active_anim = pygame.Rect(0, 0, 32, 32)

        self.state = state

        self.animations = {
                "default": {
                    "size": [64, 64],
                    "start": [0, 0],
                    "count": 1,
                    "up": 0,
                    "left": 1,
                    "down": 2,
                    "right": 3,
                    "offset": [16, 32],
                    "time": 75
                    },
                "stand": {
                    "start": [0, 512],
                    "count": 1
                    },
                "spell": {
                    "start": [0, 0],
                    "count": 7},
                "spear": {
                    "start": [0, 256],
                    "count": 8},
                "walk": {
                    "start": [0, 512],
                    "count": 9},
                "run": {
                    "start": [0, 512],
                    "count": 9,
                    "time": 50},
                "dagger": {
                    "start": [0, 768],
                    "count": 6},
                "bow": {
                    "start": [0, 1024],
                    "count": 13},
                "die": {
                    "start": [0, 1280],
                    "count": 6,
                    "up": 0,
                    "left": 0,
                    "down": 0,
                    "right": 0,
                    "time": 250,
                    },
                "dead": {
                    "start": [6 * 64, 1280],
                    "count": 1,
                    "up": 0,
                    "left": 0,
                    "down": 0,
                    "right": 0,
                    "time": 200,
                    },
                "sword": {
                    "size": [192, 192],
                    "start": [0, 1344],
                    "count": 6,
                    "offset": [80, 96]}
                }

        Animateable.__init__(self, self.pos, spritesheet, self.animations, "stand", "down")
        '''
        self.frame = 0
        self.time = 0
        '''
        self.walk_speed = 3
        self.run_speed = 5

        self.run = False
        self.speed = self.walk_speed
        self.lights = []
        self.attack = False
        self.block_time = 0

        self.reset = False

        self.add_light(lights.Light.create_light(self.pos, 128))

        self.weapon_chooser = WeaponChooser(surfaces)

        self.torch_rect = pygame.Rect(0, 0, 32, 64)
        self.torch_timer = 0
        self.torch_count = 0
        self.torch_max = 9
        self.torch_width = 32
        self.torch_pos = (142, -8)

        self.cooldown = {
                "next": 0,
                "prev": 0,
                "attack": 0,
                }

        self.attacks = {
                "dagger": [
                    pygame.Rect(0, 0, 32, 32),
                    pygame.Rect(0, 0, 32, 32),
                    ],
                "bow": [
                    pygame.Rect(0, 0, 0, 0),
                    pygame.Rect(0, 0, 0, 0)
                    ],
                "spear": [
                    pygame.Rect(0, 0, 48, 16),
                    pygame.Rect(0, 0, 16, 48),
                    ],
                "sword": [
                    pygame.Rect(0, 0, 88, 64),
                    pygame.Rect(0, 0, 80, 56),
                    ],
                }

        self.life = 90
        self.life_max = 100
        self.life_timer = 0

        self.bar_sf = pygame.Surface((600, 16))
        self.bar_sf.fill((220, 220, 220))
        self.bar_sf.fill((20, 20, 20), pygame.Rect(2, 2, 596, 12))
        
        self.bar_colors = [
                pygame.Color(206, 92, 0),
                pygame.Color(245, 121, 0),
                pygame.Color(252, 175, 62),
                pygame.Color(237, 212, 0),
                ]

        self.bar_color = choice(self.bar_colors)
        self.bar_timer = 0


    def add_goodie(self, value = 1):
        self.life += 10 * value
        if self.life > self.life_max:
            self.life = self.life_max

    def get_attack_range(self):
        if self.attack and not self.hurting():
            return self.attack_range
        return None

    def get_orientation(self):
        return 'vertical' if self.direction in ['up', 'down'] else 'horizontal'

    def draw_weapons(self, screen):
        self.weapon_chooser.draw(screen)

    def draw_bar(self, screen):
        l_m = self.life_max
        l = self.life * 596 / 100

        space = pygame.Rect(2, 2, l, 12)

        torch = self.surfaces.get("torch.png")

        sf = self.bar_sf.copy()
        sf.convert()
        #sf.fill((206, 92, 0), space)
        #sf.fill(choice(self.bar_colors), space)
        sf.fill(self.bar_color, space)
        sf.set_alpha(127)

        #screen.fill((255, 255, 255), space)
        #screen.blit(self.bar_sf, (180, 10))
        screen.blit(sf, (180, 18))
        screen.blit(torch, self.torch_pos, self.torch_rect)

    def update(self, delta):
        if self.life <= 0:
            self.state.death()
            Animateable.update(self, delta)
            return

        self.torch_timer += delta
        if self.torch_timer >= self.torch_max:
            self.torch_timer -= self.torch_max
            self.torch_count += 1
            if self.torch_count >= self.torch_max:
                self.torch_count = 0
            self.torch_rect.x = self.torch_count * self.torch_width

        self.bar_timer += delta
        if self.bar_timer > 200:
            self.bar_timer -= 200
            self.bar_color = choice(self.bar_colors)

        self.life_timer += delta
        if self.life_timer >= 750:
            self.life -= 1
            if self.get_action() == "run":
                self.life -= 1
            self.life_timer -= 750

        for cd in self.cooldown:
            self.cooldown[cd] = max(0, self.cooldown[cd] - delta)
        Collideable.update(self, delta)
        if self.hurting():
            return
        if self.attack:
            action = self.weapon_chooser.get_active_weapon()
            self.block_time += delta
            if self.block_time > self.get_animation_time(action):
                self.block_time = 0
                self.attack = False
                self.reset_animation()
        elif self.direction:
            action = "run" if self.run else "walk"
            if self.direction == "up":
                self.pos.y -= self.speed
            elif self.direction == "down":
                self.pos.y += self.speed
            elif self.direction == "left":
                self.pos.x -= self.speed
            elif self.direction == "right":
                self.pos.x += self.speed
        else:
            action = "stand"
            self.reset_animation()
        #self.animate(delta, self.last_direction, action)
        self.set_action(action)
        self.set_direction(self.last_direction)

        #for l in self.lights:
            #l.update_pos(self.pos)

        for l in self.lights:
            l.set_pos(self.pos)

        Animateable.update(self, delta)

    def hurt(self, vector):
        Collideable.hurt(self, vector)
        self.life -= 15
        self.game.play_sound("spider")

    '''
    def _get_data(self, action, prop):
        try:
            ret = self.animations[action][prop]
        except KeyError:
            ret = self.animations["default"][prop]
        return ret

    def _get_time(self, action):
        base_time = self._get_data(action, 'time')
        return base_time * self._get_data(action, 'count')

    def animate(self, delta, direction, action):
        if self.reset:
            self.frame = 0
            self.time = 0
            self.reset = False
        start = self.animations[action]["start"]
        size = self._get_data(action, 'size')
        count = self._get_data(action, 'count')
        index_y = self._get_data(action, direction)
        time = self._get_data(action, "time")

        # faster animation for running speed
        if self.speed == self.run_speed and action == "walk":
            time *= 0.6

        offset = self._get_data(action, "offset")

        self.time += delta
        if self.time > time:
            self.time -= time
            self.frame += 1

        if self.frame >= count:
            self.frame = 0
            if self.life <= 0:
                self.set_action("dead")

        self.active_anim.width, self.active_anim.height = size
        self.active_anim.x = start[0] + self.frame * size[0]
        self.active_anim.y = start[1] + index_y * size[1]

        self.draw_pos.x = self.pos.x - offset[0]
        self.draw_pos.y = self.pos.y - offset[1]
    '''

    def set_pos(self, x, y):
        self.pos.center = x, y

    def get_player_pos_ref(self):
        return self.pos

    #def hit_wall(self):
        #print "wall!"

    def draw(self, surface, viewport):
        #surface.fill((0, 255, 0), self.pos)
        #if self.attack:
            #surface.fill((255, 0, 0), self.attack_range)
        Animateable.draw(self, surface, viewport)
        #surface.blit(self.spritesheet, self.draw_pos, self.active_anim)


    def add_light(self, light):
        self.lights.append(light)

    def remove_light(self, light):
        if light in self.lights:
            self.lights.remove(light)

    def handle_events(self, events):
        if self.life <= 0:
            return
        if events.action1 or not (events.right or events.left or events.up or events.down):
            self.direction = None
        if events.action1:
            if not self.attack:

                self.game.play_sound("attack")

                self.reset = True
                self.reset_animation()
                weapon = self.weapon_chooser.get_active_weapon()
                a_r = self.attacks[weapon]
                a_r = a_r[0] if self.last_direction in ["right", "left"] else a_r[1]
                a_r.center = self.pos.center
                if self.last_direction == "up":
                    a_r.midbottom = self.pos.center
                elif self.last_direction == "left":
                    a_r.midright = self.pos.center
                elif self.last_direction == "down":
                    a_r.midtop = self.pos.center
                else:
                    a_r.midleft = self.pos.center
                self.attack_range = a_r

            self.attack = True
        if self.attack:
            return
        if events.action3 and self.cooldown["next"] == 0:
            self.cooldown["next"] = 150
            self.weapon_chooser.next_weapon()
        if events.action2 and self.cooldown["prev"] == 0:
            self.cooldown["prev"] = 150
            self.weapon_chooser.prev_weapon()
        self.speed = self.run_speed if events.run else self.walk_speed
        self.run = events.run
        if events.right:
            self.direction = "right"
            self.last_direction = "right"
        if events.left:
            self.direction = "left"
            self.last_direction = "left"
        if events.up:
            self.direction = "up"
            self.last_direction = "up"
        if events.down:
            self.direction = "down"
            self.last_direction = "down"

