import pygame

from animateable import Animateable
from collideable import Collideable
from updateable import Updateable
from drawable import Drawable

from random import choice
from random import randint

class Goodie(Collideable, Drawable):
    def __init__(self, pos, state):
        _pos = pos.copy()
        _pos.width = 22
        _pos.height = 52
        _pos.center = pos.center
        Collideable.__init__(self, _pos, state)
        self.img = state.game.get_surfaces().get("torch.png")
        self.d_r = pygame.Rect(5, 11, 24, 54)

    def draw(self, screen, viewport):
        screen.blit(self.img, self.pos, self.d_r)


class Spider(Animateable, Collideable, Updateable):
    def __init__(self, pos, surfaces, default_action = "walk", direction= "down", state = None):
        animations = {
                "default": {
                    "size": [64, 64],
                    "start": [0, 0],
                    "count": 1,
                    "up": 0,
                    "left": 1,
                    "down": 2,
                    "right": 3,
                    "offset": [8, 8],
                    "time": 100
                    },
                "stand": {
                    },
                "attack": {
                    "start": [64, 0],
                    "count": 4
                    },
                "walk": {
                    "start": [320, 0],
                    "count": 5
                    },
                "die": {
                    "start": [0, 256],
                    "count": 4,
                    "up": 0,
                    "left": 0,
                    "down": 0,
                    "right": 0
                    }
                }

        sprite = surfaces.get("spider07.png")
        pos.height, pos.width = 48, 48
        self.speed = 2
        Collideable.__init__(self, pos, state)

        Animateable.__init__(self, pos, sprite, animations, default_action, direction)

        self.state = state

        self.change_time = 0

        self.dead = False
        self.dead_time = 0
        self.dead_end = self.get_animation_time("die")
        
    def hit_wall(self, wall, direction):
        if not self.hurting():
            self.set_direction(choice(["up", "down", "right", "left"]))
        self.change_time = 0
        #if direction == "up":
            #self.pos.y += 2
        #if direction == "down":
            #self.pos.y -= 2
        #if direction == "right":
            #self.pos.x -= 2
        #if direction == "left":
            #self.pos.x += 2

    def die(self, delta):
        self.set_action = "die"
        self.dead_time += delta
        if self.dead_time >= self.dead_end:
            self.state.remove(self)
            self.state.add(Goodie(self.pos, self.state))

    def update(self, delta):
        Collideable.update(self, delta)
        Animateable.update(self, delta)
        if self.hit >= 2:
            self.die(delta)
            return

        if self.hurting():
            self.set_action("hurt")
            return
        self.change_time += delta
        if self.change_time > 2000:
            self.change_time -= 2000
            if randint(0, 5) > 3:
                self.set_direction(choice(["up", "down", "right", "left"]))

        self.set_action("walk")
        _dir = self.get_direction()
        if _dir == "up":
            self.pos.y -= self.speed
        elif _dir == "down":
            self.pos.y += self.speed
        elif _dir == "left":
            self.pos.x -= self.speed
        elif _dir == "right":
            self.pos.x += self.speed

    def draw(self, screen, viewport):
        #screen.fill((0, 255, 255), self.pos)
        Animateable.draw(self, screen, viewport)

