import game
import pygame
import pygame.gfxdraw
import pygame.locals as pgl
import random
import viewport
#from gameobjects import player
from gameobjects import animateable
import state
import gamestate

import data

from text import Text

import maploader

class StateManager(object):
    def __init__(self, game, resolution):
        self.game = game
        self.resolution = resolution
        self.active_state = MainMenu()
        self.active_state.start(self, self.resolution)
        self.states = []
        self.possible_states = {
                #'splash': Splash,
                'menu': MainMenu,
                'game': gamestate.GameState,
                'pause': PauseState,
                'highscore': None,
                'help': Help,
                'gameover': GameOver,
                }

    def replace_state(self, state):
        self.game.get_events().clear()
        self.active_state.end()
        self.active_state = state
        self.active_state.start(self, self.resolution)

    def push_state(self, state):
        self.game.get_events().clear()
        self.active_state.pause()
        self.states.append(self.active_state)
        self.active_state = state
        self.active_state.start(self, self.resolution)

    def pop_state(self):
        self.game.get_events().clear()
        state = self.active_state
        state.end()
        if len(self.states) == 0:
            self.game.quit()
            return
        self.active_state = self.states.pop()
        self.active_state.resume()

    def update(self, delta):
        self.active_state.update(delta)

    def draw(self, screen):
        self.active_state.draw(screen)

    def handle_events(self, events):
        self.active_state.handle_events(events)

    def get_state(self, state):
        return self.possible_states[state]



class Splash(state.State):
    def __init__(self):
        state.State.__init__(self)
        self.text = pygame.Rect(0, 200, 300, 80)
        self.bg = pygame.Color(240, 240, 240)
        self.color = pygame.Color(30, 30, 200)

        self.lifetime = 3500
        self.pos = pygame.Rect(0, 0, 10, 10)

    def handle_events(self, events):
        if events.get('escape'):
            self.quit()

    def start(self, manager, resolution):
        self.manager = manager
        self.resolution = resolution
        surfaces = manager.game.get_surfaces()

        self.image = surfaces.get("splash.png")
        self.pos.size = self.image.get_size()
        self.pos.left = 0

    def update(self, delta):
        if self.pos.right > self.resolution[0]:
            self.pos.x -= 2

        self.lifetime -= delta

        if self.lifetime < 0:
            self.quit()

    def draw(self, screen):
        screen.fill(self.bg)
        #screen.fill(self.color, self.text)
        screen.blit(self.image, self.pos)
        pygame.display.flip()

    def quit(self):
        state = self.manager.get_state('menu')
        self.manager.replace_state(state())


class MainMenu(state.State):
    def __init__(self):
        state.State.__init__(self)
        self.d = 0
        self.darkness_color = pygame.Color(30, 30, 90)
        self.pos = 0, 0

    def start(self, manager, resolution):
        self.d = 0
        state.State.start(self, manager, resolution)
        surfaces = manager.game.get_surfaces()
        self.bg = surfaces.get("menu.png")

        self.darkness = pygame.Surface(resolution)
        self.button_play = surfaces.get("menu_play.png")
        self.button_help = surfaces.get("menu_help.png")
        self.button_quit = surfaces.get("menu_quit.png")
        self.play_pos = pygame.Rect(576, 92, 193, 97)
        self.help_pos = pygame.Rect(576, 220, 193, 97)
        self.quit_pos = pygame.Rect(576, 412, 193, 97)

        self.play_hit = False
        self.help_hit = False
        self.quit_hit = False

    def update(self, delta):
        self.d += delta

    def resume(self):
        self.d = 0

    def draw(self, screen):
        #screen.fill((self.d, self.d, self.d))
        screen.blit(self.bg, (0, 0))

        self.darkness.fill(self.darkness_color)

        pygame.gfxdraw.filled_circle(self.darkness, self.pos[0], self.pos[1], 160, (127, 127, 127, 127))

        screen.blit(self.darkness, (0, 0), special_flags = pgl.BLEND_MULT)

        if self.play_hit:
            screen.blit(self.button_play, self.play_pos)
        if self.help_hit:
            screen.blit(self.button_help, self.help_pos)
        if self.quit_hit:
            screen.blit(self.button_quit, self.quit_pos)

        pygame.display.flip()

    def handle_events(self, events):
        if self.d < 150:
            return
        self.pos = events.mouse['x'], events.mouse["y"]
        #if events.get('escape'):
            #self.manager.pop_state()
        #if events.get('action1'):
            #state = self.manager.get_state('game')
            #self.manager.push_state(state())

        self.play_hit = self.play_pos.collidepoint(self.pos)
        self.help_hit = self.help_pos.collidepoint(self.pos)
        self.quit_hit = self.quit_pos.collidepoint(self.pos)

        if events.mouse["mouse1"]:
            if self.play_hit:
                state = self.manager.get_state('game')
                self.manager.push_state(state())
            elif self.quit_hit:
                self.manager.pop_state()
            elif self.help_hit:
                state = self.manager.get_state('help')
                self.manager.push_state(state())


class TestState(state.State):
    def __init__(self):
        state.State.__init__(self)
        self.game = game.Game.get_instance()
        self.surfaces = self.game.get_surfaces()

        self.bg = self.surfaces.get("example.png")
        self.mapdata = maploader.MapObject("test.tmx")
        self.bg = self.mapdata.get_surface()
        #self.light = self.surfaces.get("light.png")


        self.walls = self.mapdata.get_walls()

        self.light = self.surfaces.get("lights.png")
        self.light_pos = pygame.Rect(0, 0, 128, 128)
        self.light_count = 4
        self.light_flicker = 0

        self.darkness = pygame.Surface(self.game.get_resolution())
        self.darkness.fill((10, 10, 30))
        self.positioning = pygame.Rect((0, 0), self.game.get_resolution())
        self.pos = pygame.Rect(150, 150, 128, 128)

        self.pos.center = self.mapdata.get_player()
        self.player = player.Player()
        self.player.set_pos(self.pos.centerx, self.pos.centery)

        self.light_color = pygame.Color(230, 230, 230, 60)

        x, y = self.mapdata.get_dimensions()
        self.viewport = viewport.Viewport(x, y, self.game.get_resolution(), self.pos)


    def update(self, delta):
        self.light_flicker += delta
        if self.light_flicker >= 250:
            self.light_flicker -= 250
            self.light_pos.x = random.randint(0, self.light_count - 1) * 128
        self.viewport.update()
        self.player.update(delta)

    def draw(self, screen):
        screen.fill((0, 0, 0))
        vp_x = self.viewport.get_bounds().x
        vp_y = self.viewport.get_bounds().y
        screen.blit(self.bg, (0 - vp_x, 0 - vp_y))

        self.player.draw(screen)


        self.darkness.fill((10, 10, 30))
        pygame.gfxdraw.filled_circle(self.darkness, self.pos.centerx - vp_x, self.pos.centery - vp_y, 128, self.light_color)
        #self.darkness.blit(self.light, self.pos)
        self.darkness.blit(self.light, (self.pos.x - vp_x, self.pos.y -vp_y), self.light_pos)
        screen.blit(self.darkness, self.positioning, self.positioning, pgl.BLEND_RGBA_MULT)

        pygame.display.flip()

    def handle_events(self, events):
        if events.get('escape'):
            self.manager.pop_state()

        if events.get('up'):
            self.pos.y -= 5
        if events.get('down'):
            self.pos.y += 5
        if events.get('right'):
            self.pos.x += 5
        if events.get('left'):
            self.pos.x -= 5

        vp_x = self.viewport.get_bounds().x
        vp_y = self.viewport.get_bounds().y

        self.player.set_pos(self.pos.centerx - vp_x, self.pos.centery - vp_y)


class PauseState(state.State):
    def __init__(self):
        state.State.__init__(self)
        self.got_background = False
        self.bg = None

    def start(self, manager, resolution):
        self.manager = manager
        sf = manager.game.get_surfaces()
        self.resolution = resolution
        self.s_pause = sf.get("pause.png")
        self.s_quit = sf.get("quit.png")
        self.s_continue = sf.get("continue.png")
        self.quit_pos = self.s_quit.get_rect() 
        self.continue_pos = self.s_continue.get_rect() 
        self.quit_pos.topleft = self.center(self.s_quit)[0], resolution[1] / 2 + 128
        self.continue_pos.topleft = self.center(self.s_continue)[0], resolution[1] / 2
        self.start_time = 0
        self.color_time = 0
        self.color_time_max = 1500
        self.mod = pygame.Surface(resolution)
        self.color = (100, 100, 100)
        self.mod.fill(self.color)
        
    def end(self):
        self.got_background = False
        self.bg = None
        self.mod = None

    def update(self, delta):
        self.start_time += delta
        self.color_time += delta
        if self.color_time >= self.color_time_max:
            self.color_time -= self.color_time_max
            self.color = [127, 127, 127]
            self.color[random.randint(0, 2)] = 255
            self.mod.fill(self.color)

    def center(self, surface):
        return (self.resolution[0] / 2 - surface.get_width() / 2,
                self.resolution[1] / 2 - surface.get_height() / 2)

    def draw(self, screen, viewport = None):
        if not self.got_background:
            self.got_background = True
            self.bg = screen.copy()

        screen.blit(self.bg, (0, 0))
        screen.blit(self.s_pause, (self.center(self.s_pause)[0], 64))
        screen.blit(self.mod, (0, 0), special_flags = pgl.BLEND_MULT)
        screen.blit(self.s_continue, self.continue_pos)
        screen.blit(self.s_quit, self.quit_pos)

        pygame.display.flip()

    def _continue(self):
        self.manager.pop_state()

    def _quit(self):
        self.manager.pop_state()
        self.manager.pop_state()

    def handle_events(self, events):
        if self.start_time >= 250:
            if events.escape:
                self._continue()

        if events.mouse["mouse1"]:
            if self.continue_pos.collidepoint(events.mouse["x"], events.mouse["y"]):
                self._continue()
            elif self.quit_pos.collidepoint(events.mouse["x"], events.mouse["y"]):
                self._quit()


class GameOver(state.State):

    def __init__(self, score, sprite, animations):
        self.da = DeathAnimation(sprite, animations, "die")
        self.running_time = 0

        text = Text.get_instance()
        #text = Text()

        self.lose = text.render("Game Over!")
        self.score = text.render("Score: %d" % score)

        #pygame.font.init()
        #font_name = data.filepath("pressstart2p.ttf")
        #font = pygame.font.Font(font_name, 48)

        #self.lose = font.render("Game Over!", 0, (200, 200, 200))
        self.l_p = self.lose.get_rect()
        self.l_p.center = 400, 150
        #self.score = font.render("Score: %d" % score, 0, (200, 200, 200))
        self.s_p = self.score.get_rect()
        self.s_p.center = 400, 450

    def draw(self, screen, viewport = None):
        screen.fill((0, 0, 0))
        self.da.draw(screen, viewport)
        screen.blit(self.lose, self.l_p)
        screen.blit(self.score, self.s_p)
        pygame.display.flip()

    def update(self, delta):
        self.running_time += delta
        self.da.update(delta)
        
        if self.running_time > 5000:
            self.manager.pop_state()

    def handle_events(self, events):
        if self.running_time > 1000:
            if events.action1 or events.action2 or events.escape or events.run or events.mouse["mouse1"]:
                self.manager.pop_state()


class DeathAnimation(animateable.Animateable):
    def __init__(self, sprite, animations, default_action):
        pos = pygame.Rect(0, 0, 32, 32)
        pos.center = (400, 300)
        animateable.Animateable.__init__(self, pos, sprite, animations, default_action, "up")

        self.do_update = True

    def update(self, delta):
        if not self.do_update:
            return

        animateable.Animateable.update(self, delta)

    def animation_end(self, action):
        self.do_update = False
        self.frame_last()

class Help(state.State):
    def __init__(self):
        self._start = 0
        self.go = False
        self.color = [10, 10, 10]
        self.color_timer = 0
        self.texts = []
        text = Text.get_instance()

        t = text.render("Help", 32)
        self.texts.append(((30, 30), t))

        t = text.render("Gamplay:", 20)
        self.texts.append(((30, 80), t))

        t = text.render("You are trapped in a dark room with giant spiders.", 14)
        self.texts.append(((30, 110), t))

        t = text.render("Luckily you have some torches with you.", 14)
        self.texts.append(((30, 140), t))

        t = text.render("If you lose all your light, the spiders will kill you.", 14)
        self.texts.append(((30, 170), t))

        t = text.render("With ongoing time the your torches will burn up.", 14)
        self.texts.append(((30, 200), t))

        t = text.render("Kill the spiders to get more torches.", 14)
        self.texts.append(((30, 230), t))

        t = text.render("Don't touch the spiders, they will steal your torches.", 14)
        self.texts.append(((30, 260), t))

        t = text.render("If you run, your torches will burn faster, but if you", 14)
        self.texts.append(((30, 290), t))

        t = text.render("don't run, you won't be able to reach the spiders.", 14)
        self.texts.append(((30, 320), t))

        t = text.render("Controls", 20)
        self.texts.append(((30, 360), t))

        t = text.render("<Arrow keys> - Move", 14)
        self.texts.append(((30, 390), t))

        t = text.render("<Space>      - Attack", 14)
        self.texts.append(((30, 420), t))

        t = text.render("<Left Shift> - Run", 14)
        self.texts.append(((30, 450), t))

        t = text.render("<Left Ctrl>  - Previous Weapon", 14)
        self.texts.append(((30, 480), t))

        t = text.render("<Left Alt>   - Next Weapon", 14)
        self.texts.append(((30, 510), t))

        t = text.render("<Escape>     - Pause", 14)
        self.texts.append(((30, 540), t))

        t = text.render("Press <Space> to continue...", 16)
        self.texts.append(((330, 575), t))


    def update(self, delta):
        self._start += delta
        self.color_timer += delta
        if self._start > 250:
            self.go = True
        if self.color_timer > 1000:
            self.color[2] = random.choice([x * 10 for x in xrange(1, 10)])
            self.color_timer-= 1000

    def draw(self, screen, viewport = None):
        screen.fill(self.color)

        for t in self.texts:
            screen.blit(t[1], t[0])

        pygame.display.flip()

    def handle_events(self, events):
        if events.action1 or events.action2 or events.run or events.escape or events.mouse["mouse1"]:
            if self.go:
                self.manager.pop_state()

